@@ -262,7 +262,7 @@ impl<'a> Tokenizer<'a> {
 
     #[inline]
     pub fn next(&mut self) -> Result<Token<'a>, ()> {
-        next_token(self).ok_or(())
+        next_token(self)
     }
 
     #[inline]
@@ -359,17 +359,14 @@ impl<'a> Tokenizer<'a> {
     }
 
     #[inline]
-    fn next_char(&self) -> char { self.char_at(0) }
-
-    #[inline]
-    fn char_at(&self, offset: usize) -> char {
-        self.input[self.position + offset..].chars().next().unwrap()
+    fn next_char(&self) -> char {
+        self.input[self.position..].chars().next().unwrap()
     }
 
     #[inline]
     fn has_newline_at(&self, offset: usize) -> bool {
         self.position + offset < self.input.len() &&
-        matches!(self.char_at(offset), '\n' | '\r' | '\x0C')
+        matches!(self.byte_at(offset), b'\n' | b'\r' | b'\x0C')
     }
 
     #[inline]
@@ -380,8 +377,8 @@ impl<'a> Tokenizer<'a> {
     }
 
     #[inline]
-    fn starts_with(&self, needle: &str) -> bool {
-        self.input[self.position..].starts_with(needle)
+    fn starts_with(&self, needle: &[u8]) -> bool {
+        self.input.as_bytes()[self.position..].starts_with(needle)
     }
 }
 
@@ -401,164 +398,169 @@ pub struct SourceLocation {
 }
 
 
-fn next_token<'a>(tokenizer: &mut Tokenizer<'a>) -> Option<Token<'a>> {
+fn next_token<'a>(tokenizer: &mut Tokenizer<'a>) -> Result<Token<'a>, ()> {
     if tokenizer.is_eof() {
-        return None
+        return Err(())
     }
-    let c = tokenizer.next_char();
-    let token = match c {
-        '\t' | '\n' | ' ' | '\r' | '\x0C' => {
+    let b = tokenizer.next_byte_unchecked();
+    let token = match_byte! { b,
+        b'\t' | b'\n' | b' ' | b'\r' | b'\x0C' => {
             let start_position = tokenizer.position();
             tokenizer.advance(1);
             while !tokenizer.is_eof() {
-                match tokenizer.next_char() {
-                    ' ' | '\t' | '\n' | '\r' | '\x0C' => tokenizer.advance(1),
+                match tokenizer.next_byte_unchecked() {
+                    b' ' | b'\t' | b'\n' | b'\r' | b'\x0C' => tokenizer.advance(1),
                     _ => break,
                 }
             }
             WhiteSpace(tokenizer.slice_from(start_position))
         },
-        '"' => consume_string(tokenizer, false),
-        '#' => {
+        b'"' => { consume_string(tokenizer, false) },
+        b'#' => {
             tokenizer.advance(1);
             if is_ident_start(tokenizer) { IDHash(consume_name(tokenizer)) }
-            else if !tokenizer.is_eof() && match tokenizer.next_char() {
-                'a'...'z' | 'A'...'Z' | '0'...'9' | '-' | '_' => true,
-                '\\' => !tokenizer.has_newline_at(1),
-                _ => c > '\x7F',  // Non-ASCII
+            else if !tokenizer.is_eof() && match tokenizer.next_byte_unchecked() {
+                b'a'...b'z' | b'A'...b'Z' | b'0'...b'9' | b'-' | b'_' => true,
+                b'\\' => !tokenizer.has_newline_at(1),
+                _ => !b.is_ascii(),
             } { Hash(consume_name(tokenizer)) }
-            else { Delim(c) }
+            else { Delim('#') }
         },
-        '$' => {
-            if tokenizer.starts_with("$=") { tokenizer.advance(2); SuffixMatch }
-            else { tokenizer.advance(1); Delim(c) }
+        b'$' => {
+            if tokenizer.starts_with(b"$=") { tokenizer.advance(2); SuffixMatch }
+            else { tokenizer.advance(1); Delim('$') }
         },
-        '\'' => consume_string(tokenizer, true),
-        '(' => { tokenizer.advance(1); ParenthesisBlock },
-        ')' => { tokenizer.advance(1); CloseParenthesis },
-        '*' => {
-            if tokenizer.starts_with("*=") { tokenizer.advance(2); SubstringMatch }
-            else { tokenizer.advance(1); Delim(c) }
+        b'\'' => { consume_string(tokenizer, true) },
+        b'(' => { tokenizer.advance(1); ParenthesisBlock },
+        b')' => { tokenizer.advance(1); CloseParenthesis },
+        b'*' => {
+            if tokenizer.starts_with(b"*=") { tokenizer.advance(2); SubstringMatch }
+            else { tokenizer.advance(1); Delim('*') }
         },
-        '+' => {
+        b'+' => {
             if (
                 tokenizer.has_at_least(1)
-                && matches!(tokenizer.char_at(1), '0'...'9')
+                && matches!(tokenizer.byte_at(1), b'0'...b'9')
             ) || (
                 tokenizer.has_at_least(2)
-                && tokenizer.char_at(1) == '.'
-                && matches!(tokenizer.char_at(2), '0'...'9')
+                && tokenizer.byte_at(1) == b'.'
+                && matches!(tokenizer.byte_at(2), b'0'...b'9')
             ) {
                 consume_numeric(tokenizer)
             } else {
                 tokenizer.advance(1);
-                Delim(c)
+                Delim('+')
             }
         },
-        ',' => { tokenizer.advance(1); Comma },
-        '-' => {
+        b',' => { tokenizer.advance(1); Comma },
+        b'-' => {
             if (
                 tokenizer.has_at_least(1)
-                && matches!(tokenizer.char_at(1), '0'...'9')
+                && matches!(tokenizer.byte_at(1), b'0'...b'9')
             ) || (
                 tokenizer.has_at_least(2)
-                && tokenizer.char_at(1) == '.'
-                && matches!(tokenizer.char_at(2), '0'...'9')
+                && tokenizer.byte_at(1) == b'.'
+                && matches!(tokenizer.byte_at(2), b'0'...b'9')
             ) {
                 consume_numeric(tokenizer)
-            } else if tokenizer.starts_with("-->") {
+            } else if tokenizer.starts_with(b"-->") {
                 tokenizer.advance(3);
                 CDC
             } else if is_ident_start(tokenizer) {
                 consume_ident_like(tokenizer)
             } else {
                 tokenizer.advance(1);
-                Delim(c)
+                Delim('-')
             }
         },
-        '.' => {
+        b'.' => {
             if tokenizer.has_at_least(1)
-                && matches!(tokenizer.char_at(1), '0'...'9'
+                && matches!(tokenizer.byte_at(1), b'0'...b'9'
             ) {
                 consume_numeric(tokenizer)
             } else {
                 tokenizer.advance(1);
-                Delim(c)
+                Delim('.')
             }
         }
-        '/' if tokenizer.starts_with("/*") => {
-            tokenizer.advance(2);  // consume "/*"
-            let start_position = tokenizer.position();
-            let content;
-            match tokenizer.input[tokenizer.position..].find("*/") {
-                Some(offset) => {
-                    tokenizer.advance(offset);
-                    content = tokenizer.slice_from(start_position);
-                    tokenizer.advance(2);
-                }
-                None => {
-                    tokenizer.position = tokenizer.input.len();
-                    content = tokenizer.slice_from(start_position);
+        b'/' => {
+            if tokenizer.starts_with(b"/*") {
+                tokenizer.advance(2);  // consume "/*"
+                let start_position = tokenizer.position();
+                let content;
+                match tokenizer.input[tokenizer.position..].find("*/") {
+                    Some(offset) => {
+                        tokenizer.advance(offset);
+                        content = tokenizer.slice_from(start_position);
+                        tokenizer.advance(2);
+                    }
+                    None => {
+                        tokenizer.position = tokenizer.input.len();
+                        content = tokenizer.slice_from(start_position);
+                    }
                 }
+                Comment(content)
+            } else {
+                tokenizer.advance(1);
+                Delim('/')
             }
-            Comment(content)
         }
-        '0'...'9' => consume_numeric(tokenizer),
-        ':' => { tokenizer.advance(1); Colon },
-        ';' => { tokenizer.advance(1); Semicolon },
-        '<' => {
-            if tokenizer.starts_with("<!--") {
+        b'0'...b'9' => { consume_numeric(tokenizer) },
+        b':' => { tokenizer.advance(1); Colon },
+        b';' => { tokenizer.advance(1); Semicolon },
+        b'<' => {
+            if tokenizer.starts_with(b"<!--") {
                 tokenizer.advance(4);
                 CDO
             } else {
                 tokenizer.advance(1);
-                Delim(c)
+                Delim('<')
             }
         },
-        '@' => {
+        b'@' => {
             tokenizer.advance(1);
             if is_ident_start(tokenizer) { AtKeyword(consume_name(tokenizer)) }
-            else { Delim(c) }
+            else { Delim('@') }
         },
-        'u' | 'U' => {
+        b'u' | b'U' => {
             if tokenizer.has_at_least(2)
-               && tokenizer.char_at(1) == '+'
-               && matches!(tokenizer.char_at(2), '0'...'9' | 'a'...'f' | 'A'...'F' | '?')
+               && tokenizer.byte_at(1) == b'+'
+               && matches!(tokenizer.byte_at(2), b'0'...b'9' | b'a'...b'f' | b'A'...b'F' | b'?')
             { consume_unicode_range(tokenizer) }
             else { consume_ident_like(tokenizer) }
         },
-        'a'...'z' | 'A'...'Z' | '_' | '\0' => consume_ident_like(tokenizer),
-        '[' => { tokenizer.advance(1); SquareBracketBlock },
-        '\\' => {
+        b'a'...b'z' | b'A'...b'Z' | b'_' | b'\0' => { consume_ident_like(tokenizer) },
+        b'[' => { tokenizer.advance(1); SquareBracketBlock },
+        b'\\' => {
             if !tokenizer.has_newline_at(1) { consume_ident_like(tokenizer) }
-            else { tokenizer.advance(1); Delim(c) }
+            else { tokenizer.advance(1); Delim('\\') }
         },
-        ']' => { tokenizer.advance(1); CloseSquareBracket },
-        '^' => {
-            if tokenizer.starts_with("^=") { tokenizer.advance(2); PrefixMatch }
-            else { tokenizer.advance(1); Delim(c) }
+        b']' => { tokenizer.advance(1); CloseSquareBracket },
+        b'^' => {
+            if tokenizer.starts_with(b"^=") { tokenizer.advance(2); PrefixMatch }
+            else { tokenizer.advance(1); Delim('^') }
         },
-        '{' => { tokenizer.advance(1); CurlyBracketBlock },
-        '|' => {
-            if tokenizer.starts_with("|=") { tokenizer.advance(2); DashMatch }
-            else if tokenizer.starts_with("||") { tokenizer.advance(2); Column }
-            else { tokenizer.advance(1); Delim(c) }
+        b'{' => { tokenizer.advance(1); CurlyBracketBlock },
+        b'|' => {
+            if tokenizer.starts_with(b"|=") { tokenizer.advance(2); DashMatch }
+            else if tokenizer.starts_with(b"||") { tokenizer.advance(2); Column }
+            else { tokenizer.advance(1); Delim('|') }
         },
-        '}' => { tokenizer.advance(1); CloseCurlyBracket },
-        '~' => {
-            if tokenizer.starts_with("~=") { tokenizer.advance(2); IncludeMatch }
-            else { tokenizer.advance(1); Delim(c) }
+        b'}' => { tokenizer.advance(1); CloseCurlyBracket },
+        b'~' => {
+            if tokenizer.starts_with(b"~=") { tokenizer.advance(2); IncludeMatch }
+            else { tokenizer.advance(1); Delim('~') }
         },
         _ => {
-            if c > '\x7F' {  // Non-ASCII
+            if !b.is_ascii() {
                 consume_ident_like(tokenizer)
             } else {
                 tokenizer.advance(1);
-                Delim(c)
+                Delim(b as char)
             }
         },
     };
-    Some(token)
+    Ok(token)
 }
 
 
@@ -580,35 +582,47 @@ fn consume_quoted_string<'a>(tokenizer: &mut Tokenizer<'a>, single_quote: bool)
         if tokenizer.is_eof() {
             return Ok(Borrowed(tokenizer.slice_from(start_pos)))
         }
-        match tokenizer.next_byte_unchecked() {
-            b'"' if !single_quote => {
-                let value = tokenizer.slice_from(start_pos);
-                tokenizer.advance(1);
-                return Ok(Borrowed(value))
+        match_byte! { tokenizer.next_byte_unchecked(),
+            b'"' => {
+                if !single_quote {
+                    let value = tokenizer.slice_from(start_pos);
+                    tokenizer.advance(1);
+                    return Ok(Borrowed(value))
+                }
             }
-            b'\'' if single_quote => {
-                let value = tokenizer.slice_from(start_pos);
-                tokenizer.advance(1);
-                return Ok(Borrowed(value))
+            b'\'' => {
+                if single_quote {
+                    let value = tokenizer.slice_from(start_pos);
+                    tokenizer.advance(1);
+                    return Ok(Borrowed(value))
+                }
             }
             b'\\' | b'\0' => {
                 string_bytes = tokenizer.slice_from(start_pos).as_bytes().to_owned();
                 break
             }
-            b'\n' | b'\r' | b'\x0C' => return Err(()),
-            _ => {
-                tokenizer.consume_byte();
-            }
+            b'\n' | b'\r' | b'\x0C' => { return Err(()) },
+            _ => {}
         }
+        tokenizer.consume_byte();
     }
 
     while !tokenizer.is_eof() {
         if matches!(tokenizer.next_byte_unchecked(), b'\n' | b'\r' | b'\x0C') {
             return Err(());
         }
-        match tokenizer.consume_byte() {
-            b'"' if !single_quote => break,
-            b'\'' if single_quote => break,
+        let c = tokenizer.consume_byte();
+        match_byte! { c,
+            b'"' => {
+                if !single_quote {
+                    break;
+                }
+            }
+            b'\'' => {
+                if single_quote {
+                    break;
+                }
+            }
             b'\\' => {
                 if !tokenizer.is_eof() {
                     match tokenizer.next_byte_unchecked() {
@@ -624,15 +638,19 @@ fn consume_quoted_string<'a>(tokenizer: &mut Tokenizer<'a>, single_quote: bool)
                     }
                 }
                 // else: escaped EOF, do nothing.
+                continue;
             }
             b'\0' => {
                 // string.push('\u{FFFD}'),
                 string_bytes.push(0xef);
                 string_bytes.push(0xbf);
                 string_bytes.push(0xbd);
+                continue;
             }
-            c => string_bytes.push(c),
+            _ => {},
         }
+
+        string_bytes.push(c);
     }
 
     Ok(Owned(to_utf8(string_bytes)))
@@ -641,15 +659,19 @@ fn consume_quoted_string<'a>(tokenizer: &mut Tokenizer<'a>, single_quote: bool)
 
 #[inline]
 fn is_ident_start(tokenizer: &mut Tokenizer) -> bool {
-    !tokenizer.is_eof() && match tokenizer.next_char() {
-        'a'...'z' | 'A'...'Z' | '_' | '\0' => true,
-        '-' => tokenizer.has_at_least(1) && match tokenizer.char_at(1) {
-            'a'...'z' | 'A'...'Z' | '-' | '_' | '\0' => true,
-            '\\' => !tokenizer.has_newline_at(1),
-            c => c > '\x7F',  // Non-ASCII
+    !tokenizer.is_eof() && match_byte! { tokenizer.next_byte_unchecked(),
+        b'a'...b'z' | b'A'...b'Z' | b'_' | b'\0' => { true },
+        b'-' => {
+            tokenizer.has_at_least(1) && match_byte! { tokenizer.byte_at(1),
+                b'a'...b'z' | b'A'...b'Z' | b'-' | b'_' | b'\0' => {
+                    true
+                }
+                b'\\' => { !tokenizer.has_newline_at(1) }
+                b => { !b.is_ascii() },
+            }
         },
-        '\\' => !tokenizer.has_newline_at(1),
-        c => c > '\x7F',  // Non-ASCII
+        b'\\' => { !tokenizer.has_newline_at(1) },
+        b => { !b.is_ascii() },
     }
 }
 
@@ -679,14 +701,16 @@ fn consume_name<'a>(tokenizer: &mut Tokenizer<'a>) -> Cow<'a, str> {
         if tokenizer.is_eof() {
             return Borrowed(tokenizer.slice_from(start_pos))
         }
-        match tokenizer.next_byte_unchecked() {
-            b'a'...b'z' | b'A'...b'Z' | b'0'...b'9' | b'_' | b'-'  => tokenizer.advance(1),
+        match_byte! { tokenizer.next_byte_unchecked(),
+            b'a'...b'z' | b'A'...b'Z' | b'0'...b'9' | b'_' | b'-' => { tokenizer.advance(1) },
             b'\\' | b'\0' => {
                 value_bytes = tokenizer.slice_from(start_pos).as_bytes().to_owned();
                 break
             }
-            c if c.is_ascii() => return Borrowed(tokenizer.slice_from(start_pos)),
-            _ => {
+            c => {
+                if c.is_ascii() {
+                    return Borrowed(tokenizer.slice_from(start_pos));
+                }
                 tokenizer.advance(1);
             }
         }
@@ -694,7 +718,7 @@ fn consume_name<'a>(tokenizer: &mut Tokenizer<'a>) -> Cow<'a, str> {
 
     while !tokenizer.is_eof() {
         let c = tokenizer.next_byte_unchecked();
-        match c {
+        match_byte! { c,
             b'a'...b'z' | b'A'...b'Z' | b'0'...b'9' | b'_' | b'-'  => {
                 tokenizer.advance(1);
                 value_bytes.push(c)
@@ -711,10 +735,12 @@ fn consume_name<'a>(tokenizer: &mut Tokenizer<'a>) -> Cow<'a, str> {
                 value_bytes.push(0xbf);
                 value_bytes.push(0xbd);
             },
-            c if c.is_ascii() => break,
-            other => {
+            _ => {
+                if c.is_ascii() {
+                    break;
+                }
                 tokenizer.advance(1);
-                value_bytes.push(other)
+                value_bytes.push(c)
             }
         }
     }
@@ -729,9 +755,9 @@ fn consume_numeric<'a>(tokenizer: &mut Tokenizer<'a>) -> Token<'a> {
     // Do all the math in f64 so that large numbers overflow to +/-inf
     // and i32::{MIN, MAX} are within range.
 
-    let (has_sign, sign) = match tokenizer.next_char() {
-        '-' => (true, -1.),
-        '+' => (true, 1.),
+    let (has_sign, sign) = match tokenizer.next_byte_unchecked() {
+        b'-' => (true, -1.),
+        b'+' => (true, 1.),
         _ => (false, 1.),
     };
     if has_sign {
@@ -750,8 +776,8 @@ fn consume_numeric<'a>(tokenizer: &mut Tokenizer<'a>) -> Token<'a> {
     let mut is_integer = true;
 
     let mut fractional_part: f64 = 0.;
-    if tokenizer.has_at_least(1) && tokenizer.next_char() == '.'
-            && matches!(tokenizer.char_at(1), '0'...'9') {
+    if tokenizer.has_at_least(1) && tokenizer.next_byte_unchecked() == b'.'
+            && matches!(tokenizer.byte_at(1), b'0'...b'9') {
         is_integer = false;
         tokenizer.advance(1);  // Consume '.'
         let mut factor = 0.1;
@@ -769,19 +795,19 @@ fn consume_numeric<'a>(tokenizer: &mut Tokenizer<'a>) -> Token<'a> {
 
     if (
         tokenizer.has_at_least(1)
-        && matches!(tokenizer.next_char(), 'e' | 'E')
-        && matches!(tokenizer.char_at(1), '0'...'9')
+        && matches!(tokenizer.next_byte_unchecked(), b'e' | b'E')
+        && matches!(tokenizer.byte_at(1), b'0'...b'9')
     ) || (
         tokenizer.has_at_least(2)
-        && matches!(tokenizer.next_char(), 'e' | 'E')
-        && matches!(tokenizer.char_at(1), '+' | '-')
-        && matches!(tokenizer.char_at(2), '0'...'9')
+        && matches!(tokenizer.next_byte_unchecked(), b'e' | b'E')
+        && matches!(tokenizer.byte_at(1), b'+' | b'-')
+        && matches!(tokenizer.byte_at(2), b'0'...b'9')
     ) {
         is_integer = false;
         tokenizer.advance(1);
-        let (has_sign, sign) = match tokenizer.next_char() {
-            '-' => (true, -1.),
-            '+' => (true, 1.),
+        let (has_sign, sign) = match tokenizer.next_byte_unchecked() {
+            b'-' => (true, -1.),
+            b'+' => (true, 1.),
             _ => (false, 1.),
         };
         if has_sign {
@@ -810,7 +836,7 @@ fn consume_numeric<'a>(tokenizer: &mut Tokenizer<'a>) -> Token<'a> {
         None
     };
 
-    if !tokenizer.is_eof() && tokenizer.next_char() == '%' {
+    if !tokenizer.is_eof() && tokenizer.next_byte_unchecked() == b'%' {
         tokenizer.advance(1);
         return Percentage(PercentageValue {
             unit_value: value as f32 / 100.,
@@ -852,32 +878,31 @@ fn to_utf8(string_bytes: Vec<u8>) -> String {
 }
 
 fn consume_unquoted_url<'a>(tokenizer: &mut Tokenizer<'a>) -> Result<Token<'a>, ()> {
-
     for (offset, c) in tokenizer.input[tokenizer.position..].bytes().enumerate() {
-        match c {
+        match_byte! { c,
             b' ' | b'\t' | b'\n' | b'\r' | b'\x0C' => {},
-            b'"' | b'\'' => return Err(()),  // Do not advance
+            b'"' | b'\'' => { return Err(()) },  // Do not advance
             b')' => {
                 tokenizer.advance(offset + 1);
                 return Ok(UnquotedUrl(Borrowed("")));
             }
             _ => {
                 tokenizer.advance(offset);
-                return Ok(consume_unquoted_url(tokenizer))
+                return Ok(consume_unquoted_url_internal(tokenizer))
             }
         }
     }
     tokenizer.position = tokenizer.input.len();
     return Ok(UnquotedUrl(Borrowed("")));
 
-    fn consume_unquoted_url<'a>(tokenizer: &mut Tokenizer<'a>) -> Token<'a> {
+    fn consume_unquoted_url_internal<'a>(tokenizer: &mut Tokenizer<'a>) -> Token<'a> {
         let start_pos = tokenizer.position();
         let mut string_bytes: Vec<u8>;
         loop {
             if tokenizer.is_eof() {
                 return UnquotedUrl(Borrowed(tokenizer.slice_from(start_pos)))
             }
-            match tokenizer.next_byte_unchecked() {
+            match_byte! { tokenizer.next_byte_unchecked(),
                 b' ' | b'\t' | b'\n' | b'\r' | b'\x0C' => {
                     let value = tokenizer.slice_from(start_pos);
                     tokenizer.advance(1);
@@ -903,13 +928,17 @@ fn consume_unquoted_url<'a>(tokenizer: &mut Tokenizer<'a>) -> Result<Token<'a>,
             }
         }
         while !tokenizer.is_eof() {
-            match tokenizer.consume_byte() {
+            match_byte! { tokenizer.consume_byte(),
                 b' ' | b'\t' | b'\n' | b'\r' | b'\x0C' => {
                     return consume_url_end(tokenizer, Owned(to_utf8(string_bytes)));
                 }
-                b')' => break,
+                b')' => {
+                    break;
+                }
                 b'\x01'...b'\x08' | b'\x0B' | b'\x0E'...b'\x1F' | b'\x7F'  // non-printable
-                    | b'"' | b'\'' | b'(' => return consume_bad_url(tokenizer),
+                    | b'"' | b'\'' | b'(' => {
+                    return consume_bad_url(tokenizer);
+                }
                 b'\\' => {
                     if tokenizer.has_newline_at(0) {
                         return consume_bad_url(tokenizer)
@@ -923,7 +952,7 @@ fn consume_unquoted_url<'a>(tokenizer: &mut Tokenizer<'a>) -> Result<Token<'a>,
                     string_bytes.push(0xbf);
                     string_bytes.push(0xbd);
                 }
-                c => string_bytes.push(c)
+                c => { string_bytes.push(c) }
             }
         }
         UnquotedUrl(Owned(to_utf8(string_bytes)))
@@ -931,10 +960,12 @@ fn consume_unquoted_url<'a>(tokenizer: &mut Tokenizer<'a>) -> Result<Token<'a>,
 
     fn consume_url_end<'a>(tokenizer: &mut Tokenizer<'a>, string: Cow<'a, str>) -> Token<'a> {
         while !tokenizer.is_eof() {
-            match tokenizer.consume_byte() {
-                b' ' | b'\t' | b'\n' | b'\r' | b'\x0C' => (),
-                b')' => break,
-                _ => return consume_bad_url(tokenizer)
+            match_byte! { tokenizer.consume_byte(),
+                b' ' | b'\t' | b'\n' | b'\r' | b'\x0C' => {},
+                b')' => { break },
+                _ => {
+                    return consume_bad_url(tokenizer);
+                }
             }
         }
         UnquotedUrl(string)
@@ -943,10 +974,12 @@ fn consume_unquoted_url<'a>(tokenizer: &mut Tokenizer<'a>) -> Result<Token<'a>,
     fn consume_bad_url<'a>(tokenizer: &mut Tokenizer<'a>) -> Token<'a> {
         // Consume up to the closing )
         while !tokenizer.is_eof() {
-            match tokenizer.consume_byte() {
-                b')' => break,
-                b'\\' => tokenizer.advance(1), // Skip an escaped ')' or '\'
-                _ => ()
+            match_byte! { tokenizer.consume_byte(),
+                b')' => { break },
+                b'\\' => {
+                    tokenizer.advance(1); // Skip an escaped ')' or '\'
+                }
+                _ => {},
             }
         }
         BadUrl
@@ -961,7 +994,7 @@ fn consume_unicode_range<'a>(tokenizer: &mut Tokenizer<'a>) -> Token<'a> {
     let max_question_marks = 6 - hex_digits;
     let mut question_marks = 0;
     while question_marks < max_question_marks && !tokenizer.is_eof()
-            && tokenizer.next_char() == '?' {
+            && tokenizer.next_byte_unchecked() == b'?' {
         question_marks += 1;
         tokenizer.advance(1)
     }
@@ -973,8 +1006,8 @@ fn consume_unicode_range<'a>(tokenizer: &mut Tokenizer<'a>) -> Token<'a> {
     } else {
         start = hex_value;
         if tokenizer.has_at_least(1) &&
-           tokenizer.next_char() == '-' &&
-           matches!(tokenizer.char_at(1), '0'...'9' | 'A'...'F' | 'a'...'f') {
+           tokenizer.next_byte_unchecked() == b'-' &&
+           matches!(tokenizer.byte_at(1), b'0'...b'9' | b'A'...b'F' | b'a'...b'f') {
             tokenizer.advance(1);
             let (hex_value, _) = consume_hex_digits(tokenizer);
             end = hex_value;
@@ -1018,7 +1051,7 @@ fn consume_escape_and_write(tokenizer: &mut Tokenizer, bytes: &mut Vec<u8>) {
 // to not be a newline.
 fn consume_escape(tokenizer: &mut Tokenizer) -> char {
     if tokenizer.is_eof() { return '\u{FFFD}' }  // Escaped EOF
-    match tokenizer.next_byte_unchecked() {
+    match_byte! { tokenizer.next_byte_unchecked(),
         b'0'...b'9' | b'A'...b'F' | b'a'...b'f' => {
             let (c, _) = consume_hex_digits(tokenizer);
             if !tokenizer.is_eof() {
@@ -1045,6 +1078,6 @@ fn consume_escape(tokenizer: &mut Tokenizer) -> char {
             tokenizer.advance(1);
             '\u{FFFD}'
         }
-        _ => tokenizer.consume_char()
+        _ => { tokenizer.consume_char() }
     }
 }
